package forbidWordsMgr

import (
	"fmt"
	"testing"
	"time"
)

// type Persion struct {
// 	name string
// }

// 屏蔽字详细信息
func Test1(t *testing.T) {
	//启动获取屏蔽字
	refreshForbidWord()

	words, pos, exist := SensitiveWords("测试，测试")
	if exist {
		t.Log(words, pos)
	}

	t.Log("END")
}

func TestForbidWord(t *testing.T) {
	// //启动获取屏蔽字
	// refreshForbidWord()

	// //判断是否有屏蔽字
	// str := "好多花姑凉"

	// isExist := IsSensitiveWords(str)

	// fmt.Println("是否有敏感字：", isExist)

	// //处理屏蔽字
	// if isExist {
	// 	newstr := HandleSendsitiveWords(str)

	// 	fmt.Println("newStr:", newstr)
	// }

	// persionSlice := make([]*Persion, 0, 5)
	// //var persionSlice []*Persion
	// for i := 0; i < 10; i++ {
	// 	p := &Persion{
	// 		name: "append",
	// 	}
	// 	persionSlice = append(persionSlice, p)
	// 	fmt.Println(&persionSlice)
	// }

	///fmt.Println(IfServerExists())
	StartRefreshForbidWordListTread(27)
	time.Sleep(10 * time.Second)
	fmt.Println(IsSensitiveWords("g点双享器"))
	fmt.Println(IsSensitiveWords("G点双享器"))
	time.Sleep(5 * time.Hour)

}
